﻿using System.Collections;
using System.Collections.Generic;
using UnityEngine;
using UnityEngine.UI;
using Cinemachine;

public class Main : Singleton<Main> {

	[SerializeField] Transform smallBirdPrefabs;

	[SerializeField] Transform SpecialPopUp;
	[SerializeField] Transform SpecialPopUpTwo;

	[SerializeField] Sprite noEgg;
	[SerializeField] Sprite haveEgg;


	[SerializeField] public Player playerOne;
	[SerializeField] public Player playerTwo;
	[SerializeField] Slider playerOneFood;
	[SerializeField] Slider playerTwoFood;
	[SerializeField] Slider woodSlider;

	[SerializeField] Transform SmallNest;
	[SerializeField] NestControl BigNest;

	[SerializeField] CycleTrack SmallNestTrack;
	[SerializeField] CycleTrack BigNestTrack;
    
	[SerializeField] CycleTrack LittleBirdCycle;

	[SerializeField] CinemachineVirtualCamera vcam;
    [SerializeField] CinemachineBasicMultiChannelPerlin noise;

	[SerializeField] Image warning;
    
	[SerializeField] LineRenderer woodLine;

	[SerializeField] Transform popup;
	[SerializeField] Text popupText;

	float slowTimeCoolDown = 1000;
	bool IsSlowReady = true;
    
	float specialCoolDown = 0;
    bool IsSpecialReady = true;

	float WoodPointCounter = 0;

	bool warned = false;
    
	private float wood = 0;

	private int counter = 0;

	public int HomeLevel = 1;
    
	public void Warning(float level = 0.5f)
	{
		if (warned == true) { return; }
      
		var temp = warning.color;
		temp.a = level;
		warning.color = temp;
		StartCoroutine(DelayWarning());
		warned = true;
		ToBigNest();
	}
    
	IEnumerator DelayWarning()
	{      
		yield return new WaitForSeconds(1f);
		warned = false;
		var temp = warning.color;
		temp.a = 0f;
        warning.color = temp;
	}


	public void ToBigNest()
	{
      
		while (SmallNestTrack.transform.childCount >0)
		{
			SmallNestTrack.transform.GetChild(0).SetParent(BigNestTrack.transform);
		}

		SmallNestTrack.InitGO();
        BigNestTrack.InitGO();
	}
    
    
	public void ToSmallNest()
    {
		//foreach (Transform item in BigNestTrack.transform)
   //     {
			//item.SetParent(SmallNestTrack.transform);

        //}

		while( BigNestTrack.transform.childCount>0)
        {
            BigNestTrack.transform.GetChild(0).SetParent(SmallNestTrack.transform);
        }

		SmallNestTrack.InitGO();
        BigNestTrack.InitGO();
    }



	public void Noise(float amplitudeGain =5f, float frequencyGain=3f)
    {
		SoundManager.Instance.PlayVoice();
        noise.m_AmplitudeGain = amplitudeGain;
        noise.m_FrequencyGain = frequencyGain;
		StartCoroutine(DelayNoise());
    }

	IEnumerator DelayNoise()
	{
		if (noise.m_AmplitudeGain <= 0.5f)
		{
			noise.m_AmplitudeGain = 0;
			noise.m_FrequencyGain = 0;
			yield break;
		}

		yield return new WaitForSeconds(0.05f);
		noise.m_AmplitudeGain = Mathf.Lerp(noise.m_AmplitudeGain, 0f, 0.2f);
		noise.m_FrequencyGain = Mathf.Lerp(noise.m_FrequencyGain, 0f, 0.2f);
		StartCoroutine(DelayNoise());
	}
    
	public bool PlayerOneHype = false;
	public bool PlayerTwoHype = false;
    
	// Use this for initialization
	void Start () {
		noise = vcam.GetCinemachineComponent<CinemachineBasicMultiChannelPerlin>();
		playerOne.OnGetGoodFood(1f);
		playerTwo.OnGetGoodFood(1f);
	}
	
	// Update is called once per frame
	void Update () {

		counter++;
		slowTimeCoolDown--;

		specialCoolDown--;
        
		if (slowTimeCoolDown <0){
			slowTimeCoolDown = 1000;
			IsSlowReady = true;
		}

		if (specialCoolDown < 0)
        {
			specialCoolDown = 1000;
			IsSpecialReady = true;
        }


		if (playerOne.MyFood < 0.5f)
		playerOne.OnGetGoodFood(0.02f);

		if (playerTwo.MyFood < 0.5f)
		playerTwo.OnGetGoodFood(0.02f);

		if (wood < 1f)
			wood += 0.0008f;
		else{
			HomeUp();
		}




		if (counter % 500 == 0 ) { ToSmallNest(); }

		if (Input.GetKeyDown(KeyCode.J)) { Noise(); Warning(); }
		if (Input.GetKeyDown(KeyCode.N)) { ToBigNest(); }
		if (Input.GetKeyDown(KeyCode.M)) { ToSmallNest(); }

		if (Input.GetKeyDown(KeyCode.Escape)) { Application.Quit(); }
  

		playerOneFood.value = playerOne.MyFood;
		playerTwoFood.value = playerTwo.MyFood;

		woodSlider.value = wood;
		if (playerOne.MyFood  <= 0.08f)
		{
			PlayerOneHype = false;
		}

		if (playerTwo.MyFood <= 0.08f)
        {
            PlayerTwoHype = false;
        }
	}

	public void StartHype(Player player)
	{
        SoundManager.Instance.PlayVoice();
		if (player == playerOne)
		{
			PlayerOneHype = true;
			if (IsSpecialReady)
			{
				IsSpecialReady = false;
				Special();
			}
		}
		if (player == playerTwo)
        {
            PlayerTwoHype = true;
        }
	}

	public void Special()
	{
		SlowTime(5);

		StartCoroutine(DelayKillSpecial());
	}


	IEnumerator DelayKillSpecial(){
		SpecialPopUp.gameObject.SetActive(true);
		yield return new WaitForSeconds(0.5f);
		SpecialPopUpTwo.gameObject.SetActive(true);
		yield return new WaitForSeconds(5f);
		SpecialPopUp.gameObject.SetActive(false);
		SpecialPopUpTwo.gameObject.SetActive(false);
	}


	public void HomeUp(){
		wood = 0;
		HomeLevel++;
		SoundManager.Instance.PlayVoice();
		if(HomeLevel%2 ==0){
			ShowPopup("We Built A Bigger Nest");
			BigNest.GetComponent<SpriteRenderer>().sprite = haveEgg;
		}else{
			ShowPopup("Baby  Bird  Hatched  !!");
			MakeSmallBird();
		}

	}
    
	public void MakeSmallBird(){

		Transform bird = Instantiate(smallBirdPrefabs);
		bird.SetParent(BigNestTrack.transform);
		BigNest.GetComponent<SpriteRenderer>().sprite = noEgg;
		SoundManager.Instance.PlayVoice();

		SmallNestTrack.InitGO();
        BigNestTrack.InitGO();

	}

	public void UseFood(Player player, float food = 0.1f){
		if (player == playerOne)
        {
            playerOne.OnGetGoodFood(-food);
        }
		if (player == playerTwo){
			playerTwo.OnGetGoodFood(-food); 
		}
	}

	public void GetWood()
	{
		wood += 0.1f;
	}

	public void PointWood(Vector3 point)
	{
		WoodPointCounter = Time.time;
		StartCoroutine(DelayPointWood(point));

	}
	IEnumerator DelayPointWood(Vector3 point)
	{
		if (WoodPointCounter + 2 < Time.time){
			woodLine.SetPosition(1, BigNest.transform.position);
			yield break;
		}

		woodLine.SetPosition(1, point);
		yield return new WaitForSeconds(0.05f);

	}

	public void ShowPopup(string words )
	{
		popupText.text = words;
		SoundManager.Instance.PlayVoice();
		popup.gameObject.SetActive(true);
		StartCoroutine(DelayHidePopup());

	}

	IEnumerator DelayHidePopup(){
		yield return new WaitForSeconds(3);
		popup.gameObject.SetActive(false);
	}


	public void SlowTime(float lenght = 1)
	{
		if (!IsSlowReady) { return; }
		IsSlowReady = false;
		SoundManager.Instance.PlayVoice();
		TimeManager.Instance.slowdownLength = lenght;
		TimeManager.Instance.DoSlowmotion();
	}



}
