﻿using UnityEngine;

public class Singleton<T> : MonoBehaviour where T : MonoBehaviour
{
    private static T _instance;

    private static object _lock = new object();

    private static bool created = false;

    //Error code: Never used
#pragma warning disable 219
    protected virtual void Awake()
    {
        T t = Singleton<T>.Instance;
    }
#pragma warning restore 219


    public static T Instance
    {
        get
        {
            if (applicationIsQuitting)
            {
                Debug.Log("[Singleton] Instance '" + typeof(T) +
                "' already destroyed on application quit." +
                " Won't create again.");
                return _instance;
            }

            lock (_lock)
            {
                if (_instance == null && !created)
                {
                    _instance = (T)FindObjectOfType(typeof(T));

                    if (FindObjectsOfType(typeof(T)).Length > 1)
                    {
                        Debug.LogError("[Singleton] Something went really wrong " +
                        " - there should never be more than 1 singleton!" +
                        " Reopenning the scene might fix it.");
                        return _instance;
                    }

                    if (_instance == null)
                    {
                        created = true;
                        GameObject singleton = new GameObject();

                        _instance = singleton.AddComponent<T>();

                        //use base class type as object name for UnitySendMessage callback
                        _instance.gameObject.name = "~" + typeof(T).ToString();

                        Debug.Log("[Singleton] An instance of " + typeof(T) +
                        " is needed in the scene, so '" + singleton +
                        "' was created with DontDestroyOnLoad.");
                    }
                    else
                    {
                        //use base class type as object name for UnitySendMessage callback
                        _instance.gameObject.name = "~" + typeof(T).ToString();

                        Debug.Log("[Singleton] Using instance already created: " +
                        _instance.gameObject.name);
                    }


                    DontDestroyOnLoad(_instance.transform.root.gameObject);
                }

                return _instance;
            }
        }
    }

    private static bool applicationIsQuitting = false;

    public virtual void OnDestroy()
    {
        applicationIsQuitting = true;
    }
}