﻿using System.Collections;
using System.Collections.Generic;
using UnityEngine;

public enum ItemsToSpawn { Wood, Men, Poo, PooTwo ,GdFood, BadFood, PooLand } 

[System.Serializable]
public class ItemToSpawnData 
{
	public ItemsToSpawn type;
	public Transform prefabs;
	public Transform Uplimit;
	public Transform Lowlimit;
	public Transform Container;
    
}

public class SpawnControl : Singleton<SpawnControl> {

	[SerializeField] Rigidbody2D playerOne;
	[SerializeField] Rigidbody2D playerTwo;

	[SerializeField] public List<ItemToSpawnData> ItemToSpawnDatas = new List<ItemToSpawnData>();
	[SerializeField] public List<ItemToSpawnData> PoopLandedDatas = new List<ItemToSpawnData>();
	[SerializeField] public List<ItemToSpawnData> MenDatas = new List<ItemToSpawnData>();

	private int counter = 0;



	// Use this for initialization
	void Start () {
		SpawnMen(Vector3.right * 120f);
	}
    
	// Update is called once per frame
	void Update () {

		counter++;
        
		if (Input.GetKeyDown(KeyCode.Z)) { Spawn(ItemsToSpawn.Wood, Vector3.zero); }
		if (Input.GetKeyDown(KeyCode.X)) { SpawnMen( Vector3.right*30f); }
		if (Input.GetKeyDown(KeyCode.RightShift) || Main.Instance.PlayerOneHype) { OnPoopOne(); }
		if (Input.GetKeyDown(KeyCode.LeftShift) || Main.Instance.PlayerTwoHype) { OnPoopTwo(); }
		if (Input.GetKeyDown(KeyCode.V)) { Spawn(ItemsToSpawn.GdFood, Vector3.zero); }
		if (Input.GetKeyDown(KeyCode.B)) { Spawn(ItemsToSpawn.BadFood, Vector3.zero); }


		if (counter % 180 ==0) { Spawn(ItemsToSpawn.Wood, Vector3.zero); }
		if (counter % 150 == 0) { SpawnMen(Vector3.right *  Random.Range(60f,200f)  ); }
		if (counter % 100 == 0) { Spawn(ItemsToSpawn.GdFood, Vector3.zero); }
		if (counter % 520 == 0)  { Spawn(ItemsToSpawn.BadFood, Vector3.zero); }
        
	}
    
	public void SpawnMen(Vector3 v)
    {
  
		var lowLimit = GetData(ItemsToSpawn.Men).Lowlimit.position;
		var upLimit = GetData(ItemsToSpawn.Men).Uplimit.position;
		Vector3 start = new Vector3(Random.Range(lowLimit.x, upLimit.x), Random.Range(lowLimit.y, upLimit.y), GetData(ItemsToSpawn.Men).prefabs.position.z);

		Transform item = Instantiate(MenDatas[Random.Range(0,2)].prefabs);
        item.position = start;
		item.position += new Vector3(0, 0, item.position.y*0.1f);
        item.GetComponent<Rigidbody2D>().AddForce(v);
		if (GetData(ItemsToSpawn.Men).Container != null)
        {
			item.SetParent(GetData(ItemsToSpawn.Men).Container);
        }
    }
    
	public void SpawnPoopLand(Vector3 start )
	{
		Transform item = Instantiate(GetPoopLand().prefabs);
		item.position = new Vector3 (start.x , start.y, item.position.z);
		item.localScale = new Vector3(1f, Random.Range(0.2f, 0.7f), 1f);
	}
    
	public void OnPoopOne()
	{
		if (Main.Instance.playerOne.MyFood < 0.02f) { return; }

        
        
		Spawn( ItemsToSpawn.Poo, new Vector3(playerOne.velocity.x * 80f, playerOne.velocity.y * 20f, 0f));
		Main.Instance.UseFood(playerOne.GetComponent<Player>(), Main.Instance.PlayerOneHype ? 0.05f : 1f);
	}

	public void OnPoopTwo()
    {
        
		if (Main.Instance.playerTwo.MyFood < 0.02f) { return; }

		Spawn( ItemsToSpawn.PooTwo , new Vector3(playerTwo.velocity.x * 80f, playerTwo.velocity.y * 20f, 0f));
        Main.Instance.UseFood(playerTwo.GetComponent<Player>(), Main.Instance.PlayerTwoHype ? 0.05f : 1f);
    }

	public void Spawn(ItemsToSpawn type , Vector3 v)
    {
		var lowLimit = GetData(type).Lowlimit.position;
		var upLimit = GetData(type).Uplimit.position;
		Vector3 start = new Vector3(Random.Range(lowLimit.x, upLimit.x), Random.Range(lowLimit.y, upLimit.y), GetData(type).prefabs.position.z);
  
		Spawn(type, start, v);
    }

	public void Spawn(ItemsToSpawn type, Vector3 upLimit, Vector3 lowLimit ,Vector3 v )
    {
		Vector3 start = new Vector3(Random.Range(lowLimit.x, upLimit.x), Random.Range(lowLimit.y, upLimit.y), GetData(type).prefabs.position.z);

		Spawn( type, start, v);
    }

	public void Spawn(ItemsToSpawn type , Vector3 start , Vector3 v )
	{

		ItemsToSpawn prefabType;
		if (type == ItemsToSpawn.Poo || type == ItemsToSpawn.PooTwo)
		{
			prefabType = Random.Range(0f, 1f) > 0.5f ? ItemsToSpawn.Poo : ItemsToSpawn.PooTwo;
		}else{
			prefabType = type;
		}


		Transform item = Instantiate(GetData(prefabType).prefabs);
		item.position = start;
		item.GetComponent<Rigidbody2D>().AddForce(v);
		if (GetData(type).Container != null){
			item.SetParent(GetData(type).Container);
		}
	}

	public ItemToSpawnData GetData(ItemsToSpawn type )
	{
		foreach (var item in ItemToSpawnDatas)
		{
			if (item.type.ToString() == type.ToString()){
				return item;
			}
		}
		return ItemToSpawnDatas[0];
	}

	public ItemToSpawnData GetPoopLand()
	{
		return (PoopLandedDatas[ Random.Range(0,PoopLandedDatas.Count) ]);
	}

}
